/**********************************************************************
 *
 * browsefilter.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: browsefilter.cpp 238 1999-04-19 23:56:10Z rjmcnab $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.2  1999/04/19 23:56:05  rjmcnab
   Finished the gdbm metadata stuff

   Revision 1.1  1999/04/06 22:22:08  rjmcnab
   Initial revision.

 */


#include "browsefilter.h"
#include "fileutil.h"


browsefilterclass::browsefilterclass () {
  gdbmptr = NULL;

  // -- onePerQuery StartResults   integer
  FilterOption_t filtopt;
  filtopt.name = "StartResults";
  filtopt.type = FilterOption_t::integert;
  filtopt.repeatable = FilterOption_t::onePerQuery;
  filtopt.defaultValue = "1";
  filtopt.validValues.push_back("1");
  filtopt.validValues.push_back("10000");
  filterOptions["StartResults"] = filtopt;

  // -- onePerQuery EndResults     integer
  filtopt.clear();
  filtopt.name = "EndResults";
  filtopt.type = FilterOption_t::integert;
  filtopt.repeatable = FilterOption_t::onePerQuery;
  filtopt.defaultValue = "10000";
  filtopt.validValues.push_back("1");
  filtopt.validValues.push_back("10000");
  filterOptions["EndResults"] = filtopt;

  // -- onePerQuery ParentNode     string ("" will return the browsing available)
  filtopt.clear();
  filtopt.name = "ParentNode";
  filtopt.type = FilterOption_t::stringt;
  filtopt.repeatable = FilterOption_t::onePerQuery;
  filtopt.defaultValue = "";
  filterOptions["ParentNode"] = filtopt;
}

browsefilterclass::~browsefilterclass () {
}

bool browsefilterclass::init (ostream &logout) {
  outconvertclass text_t2ascii;

  if (!filterclass::init(logout)) return false;

  // get the filename for the database and make sure it exists
  gdbm_filename = filename_cat(collectdir,"index","text",collection);
#ifdef _LITTLE_ENDIAN
  gdbm_filename += ".ldb";
#else
  gdbm_filename += ".bdb";
#endif
  if (!file_exists(gdbm_filename)) {
    logout << text_t2ascii
	   << "error: gdbm database \""
	   << gdbm_filename << "\" does not exist\n\n";
    return false;
  }

  return true;
}

void browsefilterclass::filter (const FilterRequest_t &request,
				FilterResponse_t &response,
				comerror_t &err, ostream &logout) {
  int numDocs = 0;
  outconvertclass text_t2ascii;

  response.clear ();
  err = noError;
  if (gdbmptr == NULL) {
    // most likely a configuration problem
    logout << text_t2ascii 
	   << "configuration error: browsefilter contains a null gdbmclass\n\n";
    err = configurationError;
    return;
  }

  // open the database
  gdbmptr->setlogout(&logout);
  if (!gdbmptr->opendatabase (gdbm_filename)) {
    // most likely a system problem (we have already checked that the
    // gdbm database exists)
    logout << text_t2ascii
	   << "system problem: open on gdbm database \""
	   << gdbm_filename << "\" failed\n\n";
    err = systemProblem;
    return;
  }

  // get the browse parameters
  int startresults = filterOptions["StartResults"].defaultValue.getint();
  int endresults = filterOptions["EndResults"].defaultValue.getint();
  text_t parentnode = filterOptions["ParentNode"].defaultValue;
  OptionValue_tarray::const_iterator options_here = request.filterOptions.begin();
  OptionValue_tarray::const_iterator options_end = request.filterOptions.end();
  while (options_here != options_end) {
    if ((*options_here).name == "StartResults") 
      startresults = (*options_here).value.getint();
    else if ((*options_here).name == "EndResults")
      endresults = (*options_here).value.getint();
    else if ((*options_here).name == "ParentNode")
      parentnode = (*options_here).value;
    else {
      logout << text_t2ascii
	     << "warning: unknown browsefilter option \""
	     << (*options_here).name
	     << "\" ignored.\n\n";
    }

    options_here++;
  }

  // adjust topmost browsing node
  if (parentnode.empty()) parentnode = "browse";

  // get the node
  infodbclass info;
  
  if ((request.filterResultOptions & FROID) ||
      (request.filterResultOptions & FRmetadata)) {
    if (!gdbmptr->getinfo(parentnode, info)) {
      // didn't find the node
      logout << text_t2ascii
	     << "warning: lookup for node \"" << parentnode 
	     << "\" failed for browsefilter.\n\n";
    } else {
      // found the node

      // replace " with the parent node name and split the contains string
      // into the result set
      text_tset resultset;
      text_t tmptext;
      text_t &contains = info["contains"];
      text_t::iterator contains_here = contains.begin();
      text_t::iterator contains_end = contains.end();
      while (contains_here != contains_end) {
	if (*contains_here == '"') tmptext += parentnode;
	else if (*contains_here == ';') {
	  if (!tmptext.empty()) resultset.insert (tmptext);
	  tmptext.clear();
	} else tmptext.push_back(*contains_here);
	
	contains_here++;
      }
      // insert the last result in the set
      if (!tmptext.empty()) resultset.insert (tmptext);
      
      // do an intersection with the input set
      if (!request.docSet.empty()) {
	intersect (resultset, request.docSet);
      }

      // create the response
      numDocs = resultset.size();
      int resultnum = 1;
      ResultDocInfo_t resultdoc;
      text_tset::iterator result_here = resultset.begin();
      text_tset::iterator result_end = resultset.end();

      while (result_here != result_end) {
	if (resultnum > endresults) break;
	if (resultnum >= startresults) {
	  resultdoc.OID = (*result_here);
	  response.docInfo.push_back(resultdoc);
	}

	resultnum++;
	result_here++;
      }
    }
  }

  response.numDocs = numDocs;
  response.isApprox = false;
}
