/**********************************************************************
 *
 * mggdbmsource.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: mggdbmsource.cpp 238 1999-04-19 23:56:10Z rjmcnab $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.4  1999/04/19 23:56:07  rjmcnab
   Finished the gdbm metadata stuff

   Revision 1.3  1999/04/12 10:30:33  rjmcnab
   Made a little more progress.

   Revision 1.2  1999/04/12 05:21:51  rjmcnab
   Started on a mg and gdbm source.

   Revision 1.1  1999/04/12 03:40:40  rjmcnab
   Initial revision.

 */


#include "mggdbmsource.h"
#include "fileutil.h"



mggdbmsourceclass::mggdbmsourceclass () {
  gdbmptr = NULL;
  mgsearchptr = NULL;
}

mggdbmsourceclass::~mggdbmsourceclass () {
}

void mggdbmsourceclass::configure (const text_t &key, const text_tarray &cfgline) {
  if (cfgline.size() >= 1) {
    const text_t &value = cfgline[0];

    if (key == "collection") collection = value;
    else if (key == "collectdir") collectdir = value;
    else if (key == "gsdlhome") gsdlhome = value;
  }
}

bool mggdbmsourceclass::init (ostream &logout) {
  outconvertclass text_t2ascii;
  
  if (!sourceclass::init (logout)) return false;
  
  // get the collection directory name
  if (collectdir.empty()) {
    collectdir = filename_cat (gsdlhome, "collect", collection);
  }
  
  // get the filename for the database and make sure it exists
  gdbm_filename = filename_cat(collectdir,"index","text",collection);
#ifdef _LITTLE_ENDIAN
  gdbm_filename += ".ldb";
#else
  gdbm_filename += ".bdb";
#endif
  if (!file_exists(gdbm_filename)) {
    logout << text_t2ascii
	   << "error: gdbm database \""
	   << gdbm_filename << "\" does not exist\n\n";
    return false;
  }
  
  return true;
}

bool mggdbmsourceclass::get_metadata (const text_t &/*requestParams*/, const text_t &/*refParams*/, 
				      const text_tarray &fields, const text_t &OID,
				      MetadataInfo_tarray &metadata, 
				      comerror_t &err, ostream &logout) {
  outconvertclass text_t2ascii;

  metadata.erase(metadata.begin(), metadata.end());

  err = noError;
  if (gdbmptr == NULL) {
    // most likely a configuration problem
    logout << text_t2ascii 
	   << "configuration error: mggdbmsource contains a null gdbmclass\n\n";
    err = configurationError;
    return true;
  }

  // open the database
  gdbmptr->setlogout(&logout);
  if (!gdbmptr->opendatabase (gdbm_filename)) {
    // most likely a system problem (we have already checked that the
    // gdbm database exists)
    logout << text_t2ascii
	   << "system problem: open on gdbm database \""
	   << gdbm_filename << "\" failed\n\n";
    err = systemProblem;
    return true;
  }

  // get the metadata
  infodbclass info;
  if (!gdbmptr->getinfo(OID, info)) return false;

  // adjust the metadata
  text_t &contains = info["contains"];
  if (contains.empty()) info["haschildren"] = 0;
  else info["haschildren"] = 1;
  contains.clear();

  // collect together the metadata
  MetadataInfo_t this_metadata;
  text_t *pos_metadata;
  text_tarray::const_iterator fields_here = fields.begin();
  text_tarray::const_iterator fields_end = fields.end();
  while (fields_here != fields_end) {
    if (*fields_here != "contains" && 
	(pos_metadata = info.getinfo(*fields_here)) != NULL) {
      this_metadata.isRef = false;
      this_metadata.field = *fields_here;
      this_metadata.value = *pos_metadata;
      metadata.push_back(this_metadata);
    }

    fields_here++;
  }

  return true;
}

