/**********************************************************************
 *
 * browsetools.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: browsetools.cpp 213 1999-03-29 02:14:31Z sjboddie $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.2  1999/03/29 02:14:29  sjboddie

   More changes to browseaction

   Revision 1.1  1999/03/25 03:10:15  sjboddie

   new library for browse stuff

 */


#include "browsetools.h"
#include "OIDtools.h"

// sets all the required macros for displaying the navigation bar
void set_navigation_bar (displayclass &disp, cgiargsclass &args,
			 recptproto *collectproto, const text_t &cname,
			 ostream &/*logout*/) {

  text_tarray classifications;
  text_t width, navigationbar, javaimagesnavbar;
  int twidth, swidth, iwidth = 0;

  collectproto->get_all_classifications (classifications);
  text_tarray::const_iterator here = classifications.begin();
  text_tarray::const_iterator end = classifications.end();

  navigationbar = "<!-- Navigation Bar -->\n";

  // calculate width of spacers and set macro
  if (args.getintarg("v") == 0) {
    disp.expandstring ("Global", "_pagewidth_", width);
    twidth = width.getint();

    disp.expandstring ("query", "_searchwidth_", width);
    iwidth += width.getint();

    while (here != end) {
      disp.expandstring (*here, "_" + *here + "width_", width);
      iwidth += width.getint();
      here ++;
    }

    if (classifications.size() == 0) swidth = twidth - iwidth;
    else if ((twidth - iwidth) < int(classifications.size())) swidth = 2;
    else swidth = (twidth - iwidth) / classifications.size();
    disp.setmacro ("widthtspace", "Global", swidth);
  }

  navigationbar += "<nobr>\n_imagesearch_";
  javaimagesnavbar = "_javasearch_";
  here = classifications.begin();
  while (here != end) {
    if (*here == cname) 
      navigationbar += "_imagespacer__icontab" + *here + "green_";
    else 
      navigationbar += "_imagespacer__image" + *here + "_";
    javaimagesnavbar += "_java" + *here + "_";
    here ++;
  }
  navigationbar += "\n</nobr>\n";
  navigationbar += "<!-- End of Navigation Bar -->\n";
  disp.setmacro ("navigationbar", cname, navigationbar);
  if (args.getintarg("v") == 0) 
    disp.setmacro ("javaimagesnavbar", cname, javaimagesnavbar);
}


// sets the _httparrowprevious_ and _httparrownext_ macros
// used by the forward and reverse navigation arrows
void set_arrow_links (displayclass &disp, cgiargsclass &args, 
		      recptproto */*collectproto*/, const text_t &cname,
		      ostream &/*logout*/) {

  disp.setmacro ("httparrowprevious", cname, args["d"] + ".prevpage");
  disp.setmacro ("httparrownext", cname, args["d"] + ".nextpage");

}


// sets the _classificationlinks_ macro 
void set_classification_links (displayclass &disp, cgiargsclass &args, 
			       recptproto */*collectproto*/, const text_t &cname,
			       ostream &/*logout*/) {

  if ((args["cl"]).empty()) return;

  text_t link = "<a\nhref=\"_httpbrowse_&d=";
  text_tarray siblings;
  text_t child, classificationlinks;

  get_children_array (get_parent(args["cl"]), siblings);
	
  classificationlinks += "<!-- Classification Links -->\n";
  classificationlinks += "<table width=_pagewidth_ cellpadding=0 cellspacing=0 border=0>\n";
  classificationlinks += "<tr><td valign=top><center>\n";
  
  text_tarray::const_iterator here = siblings.begin();
  text_tarray::const_iterator end = siblings.end();

  while (here != end) {
    if (*here == args["cl"]) {
      classificationlinks += "\n<b>" + *here + "</b>&nbsp;&nbsp;&nbsp;";
    } else {
      get_first_child (*here, child);
      classificationlinks += link + *here + "." + child + "\">";
      classificationlinks += *here + "</a>";
      if ((here + 1) != end) classificationlinks += "&nbsp;&nbsp;&nbsp;";
    }
    here ++;
  }
  
  classificationlinks += "\n</center></td></tr></table>\n";
  classificationlinks += "<!-- End of Classification Links -->\n";
  
  disp.setmacro ("classificationlinks", cname, classificationlinks);
}


void get_hierarchy_toc (cgiargsclass &args, recptproto *collectproto, 
			text_t &toc, ostream &logout) {

  toc.clear();
  toc += "\n<!-- Table of Contents produced by browsetools::get_hierarchy_toc -->\n\n";

  toc += "<p><table width=100% cellpadding=0 cellspacing=0 border=0><tr>\n";
  toc += "<td valign=top>\n";
  
  // get table of contents
  if (args.getintarg("gc"))
    get_expanded_toc(args, collectproto, toc, logout);
  else
    get_contracted_toc(args, collectproto, toc, logout);
  
  toc += "</td></tr></table>\n";
  toc += "\n<!-- end of Table of Contents -->\n";
}


void get_book_toc (cgiargsclass &/*args*/, recptproto */*collectproto*/, 
		   text_t &/*toc*/, ostream &/*logout*/) {

}


void get_datelist_toc (cgiargsclass &/*args*/, recptproto */*collectproto*/, 
		       text_t &/*toc*/, ostream &/*logout*/) {

}


void get_list_toc (cgiargsclass &args, recptproto */*collectproto*/, 
		   text_t &toc, ostream &/*logout*/) {

  text_tarray siblings;
  toc += "\n<!-- Table of Contents produced by browsetools::get_list_toc -->\n\n";

  // load up siblings array
  if ((args["d"]).empty()) get_children_array (get_parent(args["cl"]), siblings);
  else if (is_top (args["d"])) get_children_array (args["cl"], siblings);
  else get_children_array (get_parent(args["d"]), siblings);

  text_t section, child;
  text_t doclink = "<a href=\"_httptext_&d=";
  toc += "<p>\n";
  
  text_tarray::const_iterator thissection = siblings.begin();
  text_tarray::const_iterator end = siblings.end();
  
  while (thissection != end) {
    
    section.clear();
    child.clear();

    get_first_child(*thissection, child);

    toc += "<br>" + doclink + *thissection + "." + child + 
      ".1\">" + *thissection + " title</a>\n";
    thissection ++;
  }
  toc += "\n<!-- end of Table of Contents -->\n";
}


void get_contracted_toc (cgiargsclass &args, recptproto */*collectproto*/,
			 text_t &return_text, ostream &/*logout*/) {
  
  int colnum, tabcount = 0;
  text_tarray parents, siblings;
  
  // load up the parents array
  // don't want classifications unless at top level contents
  if (!(args["cl"]).empty() && is_top(args["d"]))
    get_parents_array (args["cl"] + ".", parents); 
  get_parents_array (args["d"], parents);

  // load up siblings array
  if ((args["d"]).empty()) get_children_array (get_parent(args["cl"]), siblings);
  else if (is_top (args["d"])) get_children_array (args["cl"], siblings);
  else get_children_array (get_parent(args["d"]), siblings);

  return_text += "<table>\n";
  colnum = parents.size() + 1;

  // sort out toc of sections parents
  if (parents.size() > 0) 
    get_parents_toc(args, parents, tabcount, colnum, return_text);
  
  // sort out toc of sections siblings
  if (siblings.size() > 0) 
    get_siblings_toc (args, siblings, tabcount, colnum, return_text);
		      
  return_text += "</table>\n";
}


void get_expanded_toc (cgiargsclass &/*args*/, recptproto */*collectproto*/,
		       text_t &/*return_text*/, ostream &/*logout*/) {

  /*
  text_t doclink, icon, pointer, tab, pagetype;
  int totalcols, tabcols, tabsleft;
  text_tarray contents;
  
  // get array of all contents to be included (all contents of entire book)
  text_t book_top;
  get_book_top (booksection, book_top);
  get_contents (book_top, gdbm, contents, totalcols);
  
  vector<text_t>::const_iterator section = contents.begin();
  vector<text_t>::const_iterator end = contents.end();
  
  int count = 1;
  while (section != end) {
    tab.clear();
    gdbm.getinfo(*section, info);
    text_t icontabs;

    // set up icon
    icon = "_iconsmalltext_";
    if (is_top_level(*section)) icon = "_iconopenbook_";
    else if (!info.contents.empty()) icon = "_iconopenfolder_";

    // set up pointer
    if (*section == booksection) pointer = "_iconpointer_";
    else pointer = "_icontab_";

    // set up tabbing
    tabcols = count_dots(*section);
    for (int i = 0; i < (tabcols - 1); i++) {
      icontabs += "_icontab_";
    }
    //char *tabcolsstr;
    //itoa(tabcols, tabcolsstr, 10);
    tab = "<td valign=top colspan=" + text_t(tabcols);
    tab += ">" + icontabs + pointer + "</td>";
    tabsleft = totalcols - tabcols;
    
    // set up url
    if (is_book(*section) && !is_top_level(*section))	
      doclink = "<a href=\"_httptext_";
    else doclink = "<a href=\"_httpbrowse_";
    if (args["x"] == "1") doclink += "&x=1";
    
    return_text += "<tr>" + tab + "<td valign=top>";
    if (is_top_level(*section) && args["x"] == "1") {
      return_text += "<a href=\"\" onClick = \"close_detach()";
    } else {
      if (args["g"][1] == '0' || is_top_level(*section) ||
	  !are_same_chapter(booksection, *section) ||
	  !info.contents.empty()) {
	return_text += doclink + "d=" + classification;
	return_text += "." + *section + "&a=" + pagetype;
      } else {
	return_text += "<a href=\"#";
	return_text += count;
	count ++;
      }
    }
    return_text += "\">" + icon + "</a></td>";
    return_text += "<td colspan=" + text_t(tabsleft);
    return_text += ">" + info.title;
    return_text += "</td></tr>\n";
    
    section ++;
  }
  return_text += "</table>\n";
  return_text += "</td></tr></table>\n";
  */
}


void get_parents_toc (cgiargsclass &args, const text_tarray &parents,
		      int &tabcount, int &colnum, text_t &return_text) {  

  text_t section, tab, icon, doclink;

  doclink = "<a href=\"_httpbrowse_";
  if (args["x"] == "1") doclink += "&x=1";

  text_tarray::const_iterator thisparent = parents.begin();
  text_tarray::const_iterator end = parents.end();

  while (thisparent != end) {
    text_t icontabs;
    section.clear();
    tab.clear();
    
    // set up icon for this parent
    icon = "_iconopenfolder_";
    if (is_classification(*thisparent)) icon = "_iconopenbookshelf_";
    else if (is_top(*thisparent)) icon = "_iconopenbook_";
    
    for (int j = 0; j < tabcount; j++)
      icontabs += "_icontab_";
     
    if (!icontabs.empty()) {
      tab = "<td valign=top";
      if (tabcount > 1) tab += " colspan=" + text_t(tabcount);
      tab += ">" + icontabs + "</td>";
    }
    tabcount ++;
    
    if (is_top(*thisparent) && args["x"] == "1") {
      return_text += "<tr>" + tab + "<td valign=top><a href=\"\" ";
      return_text += "onClick = \"close_detach()\">" + icon + "</a></td>";
    } else {
      if (is_classification (*thisparent))
	return_text += "<tr>" + tab + "<td valign=top>" + doclink + "&cl=" + *thisparent;
      else
	// this may not need cl= bit once compressed args is done
	return_text += "<tr>" + tab + "<td valign=top>" + doclink + "&cl=" + args["cl"] + "&d=" + *thisparent;
      return_text += "\">" + icon + "</a></td>";
    }
    return_text += "<td"; 
    if (colnum > 1) return_text += " colspan=" + text_t(colnum);
    return_text += ">" + *thisparent + " title ";
    return_text += " by " + *thisparent + " author ";
    return_text += "</td></tr>\n";
    colnum --;
    thisparent ++;
  }
}


void get_siblings_toc (cgiargsclass &args, const text_tarray &siblings, 
		       int &tabcount, int &colnum, text_t &return_text) {
  
  text_t tab, icon, pointer, child, doclink;
  int count = 1;

  doclink = "<a href=\"_httpbrowse_";
  if (args["x"] == "1") doclink += "&x=1";
  
  text_tarray::const_iterator thissibling = siblings.begin();
  text_tarray::const_iterator sibend = siblings.end();
  
  while (thissibling != sibend) {

    // set up icon for this sibling
    icon = "_iconclosedfolder_";
    if (is_classification(*thissibling)) icon = "_iconclosedbookshelf_";
    else if (is_top(*thissibling)) icon = "_iconclosedbook_";
    
    // fit pointer into tabbing if current section
    pointer = "_icontab_";
    if (is_classification(*thissibling)) {
      if (*thissibling == args["cl"])
	pointer = "_iconpointer_";
    } else if (*thissibling == args["d"]) pointer = "_iconpointer_";
    
    text_t thesetabs;
    for (int j = 0; j < (tabcount - 1); j++)
      thesetabs += "_icontab_";
    
    tab = "<td valign=top";
    if (tabcount > 1) tab += " colspan=" + text_t(tabcount);
    tab += ">" + thesetabs + pointer + "</td>";
    
    if (contains_text(*thissibling)) {
      // sibling is text document
      icon = "_iconsmalltext_";
      
      return_text += "<tr>" + tab + "<td valign=top>";
      if (args["g"][1] == '0') {
	return_text += doclink + "&d=" + *thissibling;
      } else {
	return_text += "<a href=\"#";
	return_text += count;
	count ++;
      }
      return_text += "\">" + icon + "</a></td><td";
      if (colnum > 1) return_text += " colspan=" + text_t(colnum);
      return_text += ">" + *thissibling + " title";
      return_text += " by " + *thissibling + " author";
      return_text += "</td></tr>\n";
    } else {
      // sibling is closed book or folder so clicking 
      // it should open contents
      get_first_child(*thissibling, child);
      
      return_text += "<tr>" + tab + "<td valign=top>" + doclink + "&d=" + child;
      return_text += "\">" + icon + "</a></td><td";
      if (colnum > 1) return_text += " colspan=" + text_t(colnum);
      return_text += ">" + *thissibling + " title";
      return_text += " by " + *thissibling + " author";
      return_text += "</td></tr>\n";
    }
    thissibling ++;
  }  
}
