/**********************************************************************
 *
 * comtypes.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: comtypes.cpp 241 1999-04-21 05:23:15Z sjboddie $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.7  1999/04/21 05:23:14  sjboddie

   Changed the way metadata is returned

   Revision 1.6  1999/04/06 22:20:34  rjmcnab
   Got browsefilter working.

   Revision 1.5  1999/03/31 23:44:47  rjmcnab
   Altered the protocol so that the metadata is part of the filter.

   Revision 1.4  1999/03/11 00:11:19  rjmcnab
   Added a function to get a string version of comerror_t

   Revision 1.3  1999/03/09 21:00:47  rjmcnab
   Reorganised the statusaction, added more functions to comtypes.

   Revision 1.2  1999/03/04 21:19:15  rjmcnab

   Added clear() functions to all the data types.

   Revision 1.1  1999/02/21 22:35:25  rjmcnab

   Initial revision.

 */


#include "comtypes.h"


text_t get_comerror_string (comerror_t err) {
  if (err == noError) return "no error";
  else if (err == authenticationFailure) return "authentication failure";
  else if (err == protocolError) return "protocol error";
  else if (err == configurationError) return "configuration error";
  else if (err == systemProblem) return "system problem";

  return "unknown problem";
}


void ShortColInfo_t::clear() {
  name.clear();
  host.clear();
  port = 0;
}

void ColInfoResponse_t::clear () {
  shortInfo.clear();
  isPublic=false;
  isBeta=false;
  buildDate=0;
  numDocs=0;
  numWords=0;
  numBytes=0;
}

void InfoFiltersResponse_t::clear () {
  filterNames.erase(filterNames.begin(), filterNames.end());
}


void InfoFilterOptionsRequest_t::clear () {
  filterName.clear();
}


void FilterOption_t::clear () {
  name.clear();
  type = booleant;
  repeatable = onePerQuery;
  defaultValue.clear();
  validValues.erase(validValues.begin(), validValues.end());
}

void FilterOption_t::check_defaultValue () {
  text_tarray::iterator here, end;

  // how the default is interpreted depends on the option type
  switch (type) {
  case booleant: 
  case enumeratedt: // has to be one of the validValues
    here = validValues.begin ();
    end = validValues.end ();
    while (here != end) {
      if (*here == defaultValue) return;
      here++;
    }
    
    break;

  case integert: // has to be in the validValues range
    if ((validValues.size() >= 2) &&
	(validValues[0].getint() <= defaultValue.getint()) &&
	(validValues[1].getint() >= defaultValue.getint()))
      return;
    break;

  case stringt: // any value is valid
    return; 
  }

  // did not find the defaultValue
  if (validValues.empty()) defaultValue.clear();
  else defaultValue = validValues[0];
}


void InfoFilterOptionsResponse_t::clear () {
  filterOptions.erase(filterOptions.begin(), filterOptions.end());
}


void OptionValue_t::clear () {
  name.clear();
  value.clear();
}


void FilterRequest_t::clear () {
  filterName.clear();
  filterOptions.erase(filterOptions.begin(), filterOptions.end());
  docSet.erase(docSet.begin(), docSet.end());
  filterResultOptions = 0;
  requestParams.clear();
  refParams.clear();
  fields.erase(fields.begin(), fields.end());
}


void TermInfo_t::clear () {
  term.clear();
  freq = 0;
  matchTerms.erase (matchTerms.begin(), matchTerms.end());
}


void MetadataInfo_t::clear () {
  params.clear();
  isRef = false;
  values.erase(values.begin(), values.end());
}


void ResultDocInfo_t::clear () {
  OID.clear ();
  ranking = 0;
  docFreq.erase(docFreq.begin(), docFreq.end());
  metadata.erase(metadata.begin(), metadata.end());
}

ResultDocInfo_t &ResultDocInfo_t::operator=(const ResultDocInfo_t &x) {
  OID=x.OID; 
  ranking=x.ranking; 
  docFreq=x.docFreq; 
  metadata=x.metadata;
  return *this;
}


void FilterResponse_t::clear () {
  numDocs = 0;
  isApprox = false;
  termInfo.erase (termInfo.begin(), termInfo.end());
  docInfo.erase (docInfo.begin(), docInfo.end());
}

