/**********************************************************************
 *
 * nullproto.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: nullproto.cpp 220 1999-03-31 23:44:49Z rjmcnab $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.4  1999/03/31 23:44:48  rjmcnab
   Altered the protocol so that the metadata is part of the filter.

   Revision 1.3  1999/03/03 23:26:35  sjboddie

   Implemented more of the protocol

   Revision 1.2  1999/02/25 21:58:58  rjmcnab

   Merged sources.

   Revision 1.1  1999/02/21 22:35:22  rjmcnab

   Initial revision.

 */


#include "nullproto.h"
#include "colservrconfig.h"
#include <assert.h>


// this configure will configure each of the collection servers
void nullproto::configure (const text_t &key, const text_tarray &cfgline) {
  // the naming of the collection should not be done here,
  // it should be done just after the collection server has been 
  // created
  if (key == "collection") return;

  collectservermapclass::iterator here = cservers.begin();
  collectservermapclass::iterator end = cservers.end();

  while (here != end) {
    assert ((*here).second.c != NULL);
    if ((*here).second.c != NULL) {
      (*here).second.c->configure (key, cfgline);
    }

    here++;
  }
}

// this init will configure and init each of the collection servers
bool nullproto::init (ostream &logout) {
  collectservermapclass::iterator here = cservers.begin();
  collectservermapclass::iterator end = cservers.end();

  while (here != end) {
    assert ((*here).second.c != NULL);
    if ((*here).second.c != NULL) {
      const colservrconf &configinfo = (*here).second.c->get_configinfo ();

      // configure this collection server
      if (!collect_cfg_read (*((*here).second.c), configinfo.gsdlhome,
			     configinfo.collection)) {
	outconvertclass text_t2ascii;
	logout << text_t2ascii 
	       << "Error: couldn't read collect.cfg file for collection \""
	       << configinfo.collection << "\", gsdlhome=\"" 
	       << configinfo.gsdlhome << "\"\n";
	return false;
      }

      if (!build_cfg_read (*((*here).second.c), configinfo.gsdlhome,
			   configinfo.collection)) {
	outconvertclass text_t2ascii;
	logout << text_t2ascii 
	       << "Error: couldn't read build.cfg file for collection \""
	       << configinfo.collection << "\", gsdlhome=\""
	       << configinfo.gsdlhome << "\"\n";
	return false;
      }

      if (!(*here).second.c->init (logout)) return false;
    }
    here++;
  }

  return true;
}

text_t nullproto::get_protocol_name () {
  return "nullproto";
}


void nullproto::get_collection_list (text_tarray &collist, comerror_t &err, 
				     ostream &/*logout*/) {
  collist.erase(collist.begin(),collist.end());
  err = noError;

  collectservermapclass::iterator here = cservers.begin();
  collectservermapclass::iterator end = cservers.end();
  while (here != end) {
    assert ((*here).second.c != NULL);
    if ((*here).second.c != NULL) {
      collist.push_back ((*here).second.c->get_collection_name());
    }
    here++;
  }
}

void nullproto::has_collection (const text_t &collection, bool &hascollection, 
				comerror_t &err, ostream &/*logout*/) {
  hascollection = (cservers.getcollectserver(collection) != NULL);
  err = noError;
}

void nullproto::ping (const text_t &collection, bool &wassuccess, 
		      comerror_t &err, ostream &/*logout*/) {
  wassuccess = (cservers.getcollectserver(collection) != NULL);
  err = noError;
}

void nullproto::get_collectinfo (const text_t &collection, 
				 ColInfoResponse_t &collectinfo,
				 comerror_t &err, ostream &logout) {
  collectserver *cserver = cservers.getcollectserver (collection);
  if (cserver != NULL) cserver->get_collectinfo (collectinfo, err, logout);
  else err = protocolError;
}


void nullproto::get_filterinfo (const text_t &collection,
				InfoFiltersResponse_t &response,
				comerror_t &err, ostream &logout) {
  collectserver *cserver = cservers.getcollectserver (collection);
  if (cserver != NULL) cserver->get_filterinfo (response, err, logout);
  else err = protocolError;
}

void nullproto::get_filteroptions (const text_t &collection,
				   const InfoFilterOptionsRequest_t &request,
				   InfoFilterOptionsResponse_t &response, 
				   comerror_t &err, ostream &logout) {
  collectserver *cserver = cservers.getcollectserver (collection);
  if (cserver != NULL) cserver->get_filteroptions (request, response, err, logout);
  else err = protocolError;
}

void nullproto::filter (const text_t &collection,
			const FilterRequest_t &request,
			FilterResponse_t &response,
			comerror_t &err, ostream &logout) {
  collectserver *cserver = cservers.getcollectserver (collection);
  if (cserver != NULL) cserver->filter (request, response, err, logout);
  else err = protocolError;
}

